//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/List/InstrumentsQListView.cpp
//! @brief     Implements class InstrumentsQListView.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/List/InstrumentsQListView.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Model/Device/InstrumentXML.h"
#include "GUI/Model/Device/InstrumentsSet.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/Layout/ApplicationSettings.h"
#include "GUI/View/List/InstrumentsQModel.h"
#include "GUI/View/Setup/FrameActions.h"
#include "GUI/View/Widget/FileDialog.h"
#include "GUI/View/Widget/ItemViewOverlayButtons.h"
#include "GUI/View/Widget/ListItemDelegate.h"
#include <QMessageBox>

InstrumentsQListView::InstrumentsQListView()
    : m_model(new InstrumentsQModel)
{
    setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
    setMovement(QListView::Static);
    setSelectionMode(QAbstractItemView::NoSelection);

    setModel(m_model);
    setItemDelegate(new ListItemDelegate(this));

    connect(gActions->new_gisas_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::addNewGISASInstrument);

    connect(gActions->new_offspec_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::addNewOffspecInstrument);

    connect(gActions->new_specular_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::addNewSpecularInstrument);

    connect(gActions->new_depthprobe_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::addNewDepthprobeInstrument);

    connect(gActions->remove_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::removeInstrument);

    connect(gActions->copy_instrument, &QAction::triggered, m_model,
            &InstrumentsQModel::copyInstrument);

    connect(gActions->store_in_library_instrument, &QAction::triggered, this,
            &InstrumentsQListView::onStoreInLibrary);

    connect(gActions->load_from_library_instrument, &QAction::triggered, this,
            &InstrumentsQListView::onLoadFromLibrary);

    connect(this, &QListView::clicked, [](const QModelIndex& qi) {
        gDoc->instrumentsModifier()->setCurrentIndex(qi.row());
        emit gDoc->instruments()->setChanged();
    });

    disconnect(gDoc->instruments(), &InstrumentsSet::setChanged, nullptr, nullptr);
    connect(gDoc->instruments(), &InstrumentsSet::setChanged, [this] {
        setCurrentIndex(m_model->index(gDoc->instruments()->currentIndex(), 0));
        repaint();
    });

    setContextMenuPolicy(Qt::ActionsContextMenu);
}

InstrumentsQListView::~InstrumentsQListView()
{
    disconnect(this, nullptr, nullptr, nullptr);
    delete m_model;
}

QSize InstrumentsQListView::sizeHint() const
{
    return {170, 400};
}

QSize InstrumentsQListView::minimumSizeHint() const
{
    return {96, 200};
}

void InstrumentsQListView::onStoreInLibrary() const
{
    const InstrumentItem* t = gDoc->instruments()->currentItem();
    if (!t)
        return;

    QString fname = GUI::Dialog::fileSaveDialog("Save instrument", appSettings->xml_dir,
                                                "XML Files (*.xml)", t->instrumentName() + ".xml");
    if (fname.isEmpty())
        return;

    try {
        InstrumentXML::save(fname, t);
    } catch (const std::exception& ex) {
        QMessageBox(QMessageBox::Warning, "BornAgain: failed saving", ex.what(), QMessageBox::Ok,
                    nullptr)
            .exec();
    }
}

void InstrumentsQListView::onLoadFromLibrary()
{
    QString fname =
        GUI::Dialog::fileOpenDialog("Load instrument", appSettings->xml_dir, "XML Files (*.xml)");
    if (fname.isEmpty())
        return;

    try {
        if (InstrumentItem* t = InstrumentXML::load(fname))
            m_model->pushInstrument(t);
    } catch (const std::exception& ex) {
        QMessageBox(QMessageBox::Warning, "BornAgain: failed loading", ex.what(), QMessageBox::Ok,
                    nullptr)
            .exec();
    }
}
