//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/List/InstrumentsQModel.cpp
//! @brief     Implements class InstrumentsQModel.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/List/InstrumentsQModel.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Model/Device/InstrumentsSet.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include <QColor>
#include <QUuid>

InstrumentsQModel::InstrumentsQModel() {}

int InstrumentsQModel::rowCount(const QModelIndex&) const
{
    return gDoc->instruments()->instrumentItems().size();
}

QVariant InstrumentsQModel::data(const QModelIndex& index, int role) const
{
    const InstrumentsSet* set = gDoc->instruments();
    if (!index.isValid())
        return {};
    size_t row = index.row();
    if (row >= set->size())
        return {};
    const InstrumentItem* t = set->at(row);

    switch (role) {
    case Qt::DisplayRole:
        return t->instrumentName();
    case Qt::BackgroundRole:
        return row == set->currentIndex() ? QColor(Qt::green) : QColor(Qt::magenta);
    default:
        return {};
    }
}

QModelIndex InstrumentsQModel::addNewGISASInstrument()
{
    auto* t = new GISASInstrumentItem;
    t->setInstrumentName("GISAS");
    return pushInstrument(t);
}

QModelIndex InstrumentsQModel::addNewOffspecInstrument()
{
    auto* t = new OffspecInstrumentItem;
    t->setInstrumentName("Offspec");
    return pushInstrument(t);
}

QModelIndex InstrumentsQModel::addNewSpecularInstrument()
{
    auto* t = new SpecularInstrumentItem;
    t->setInstrumentName("Specular");
    return pushInstrument(t);
}

QModelIndex InstrumentsQModel::addNewDepthprobeInstrument()
{
    auto* t = new DepthprobeInstrumentItem;
    t->setInstrumentName("Depthprobe");
    return pushInstrument(t);
}

void InstrumentsQModel::removeInstrument()
{
    InstrumentsSet* set = gDoc->instrumentsModifier();
    const InstrumentItem* t = set->currentItem();

    const int row = set->index_of(t);
    beginRemoveRows({}, row, row);
    set->delete_element(t);
    endRemoveRows();

    emit set->setChanged();
}

QModelIndex InstrumentsQModel::copyInstrument()
{
    InstrumentsSet* set = gDoc->instrumentsModifier();
    const InstrumentItem* t = set->currentItem();
    InstrumentItem* t2 = t->createItemCopy();
    t2->setId(QUuid::createUuid().toString());
    return pushInstrument(t2);
}

QModelIndex InstrumentsQModel::pushInstrument(InstrumentItem* t)
{
    InstrumentsSet* set = gDoc->instrumentsModifier();

    const int row = set->instrumentItems().size();
    beginInsertRows({}, row, row);
    set->push_back(t);
    endInsertRows();
    emit set->setChanged();

    return createIndex(row, 0);
}
