//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Views/InstrumentView.cpp
//! @brief     Implements class InstrumentView.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Views/InstrumentView.h"
#include "Base/Util/Assert.h"
#include "GUI/Model/Device/InstrumentItems.h"
#include "GUI/Model/Device/InstrumentsSet.h"
#include "GUI/Model/Project/ProjectDocument.h"
#include "GUI/View/Instrument/DepthprobeInstrumentEditor.h"
#include "GUI/View/Instrument/GISASInstrumentEditor.h"
#include "GUI/View/Instrument/OffspecInstrumentEditor.h"
#include "GUI/View/Instrument/SpecularInstrumentEditor.h"
#include "GUI/View/Layout/ApplicationSettings.h"
#include "GUI/View/List/InstrumentsQListView.h"
#include "GUI/View/Setup/FrameActions.h"
#include "GUI/View/Widget/GroupBoxes.h"
#include "GUI/View/Widget/StyledToolbar.h"
#include <QFormLayout>
#include <QLineEdit>
#include <QTextEdit>
#include <iostream>

InstrumentView::InstrumentView()
    : m_listing(new InstrumentsQListView)
    , m_scroll_area(new QScrollArea)
{
    auto* layout = new QVBoxLayout(this);

    // Top toolbar with action buttons "new ...instrument" etc
    auto* toolbar = new StyledToolbar;
    layout->addWidget(toolbar);
    toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

    toolbar->addAction(gActions->new_gisas_instrument);
    toolbar->addAction(gActions->new_offspec_instrument);
    toolbar->addAction(gActions->new_specular_instrument);
    toolbar->addAction(gActions->new_depthprobe_instrument);
    toolbar->addAction(gActions->remove_instrument);
    toolbar->addAction(gActions->copy_instrument);
    toolbar->addAction(gActions->store_in_library_instrument);
    toolbar->addAction(gActions->load_from_library_instrument);

    // Everything below
    auto* hLayout = new QHBoxLayout;
    layout->addLayout(hLayout);

    // Left margin: instrument list

    hLayout->addWidget(m_listing);

    // Large widget: current instrument

    hLayout->addWidget(m_scroll_area);
    m_scroll_area->setWidgetResizable(true);
    m_scroll_area->setMinimumWidth(1000);
    m_scroll_area->setWidget(new QWidget(m_scroll_area)); // initial state: blank widget

    hLayout->addStretch(1);

    //... Finalize

    connect(gDoc->instruments(), &InstrumentsSet::setChanged, this,
            &InstrumentView::createWidgetsForCurrentInstrument);

    updateActions();
}

void InstrumentView::createWidgetsForCurrentInstrument()
{
    updateActions();

    InstrumentItem* currentInstrument = gDoc->instrumentsModifier()->currentItem();
    if (!currentInstrument) {
        m_scroll_area->setWidget(new QWidget(m_scroll_area)); // blank widget
        return;
    }

    auto* w = new QWidget(m_scroll_area);
    auto* layout = new QVBoxLayout(w);

    //... Groupbox with instrument name and description (same layout for all instrument types)

    auto title = QString("Summary (%1 instrument)").arg(currentInstrument->instrumentType());
    auto* g = new CollapsibleGroupBox(title, m_scroll_area, currentInstrument->expandInfo);
    g->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    auto* formLayout = new QFormLayout;
    g->body()->setLayout(formLayout);
    layout->addWidget(g);

    auto* nameEdit = new QLineEdit(g);
    nameEdit->setText(currentInstrument->instrumentName());
    connect(nameEdit, &QLineEdit::textEdited, [this] { m_listing->repaint(); });
    formLayout->addRow("Name:", nameEdit);

    auto* descriptionEdit = new QTextEdit(g);
    descriptionEdit->setMinimumWidth(300);
    descriptionEdit->setFixedHeight(60); // TODO replace by 2*line_height
    descriptionEdit->setAcceptRichText(false);
    descriptionEdit->setTabChangesFocus(true);
    descriptionEdit->setPlainText(currentInstrument->description());
    connect(descriptionEdit, &QTextEdit::textChanged, [this] { m_listing->repaint(); });
    formLayout->addRow("Description:", descriptionEdit);

    //... All remaining content depends on instrument type

    IComponentEditor* editor;
    if (auto* ii = dynamic_cast<SpecularInstrumentItem*>(currentInstrument))
        editor = new SpecularInstrumentEditor(ii);
    else if (auto* ii = dynamic_cast<OffspecInstrumentItem*>(currentInstrument))
        editor = new OffspecInstrumentEditor(ii);
    else if (auto* ii = dynamic_cast<GISASInstrumentItem*>(currentInstrument))
        editor = new GISASInstrumentEditor(ii);
    else if (auto* ii = dynamic_cast<DepthprobeInstrumentItem*>(currentInstrument))
        editor = new DepthprobeInstrumentEditor(ii);
    else
        ASSERT_NEVER;

    layout->addWidget(editor);
    m_scroll_area->setWidget(w);
}

void InstrumentView::updateActions()
{
    bool enabled = gDoc->instruments()->currentIndex() != size_t(-1);
    gActions->remove_instrument->setEnabled(enabled);
    gActions->copy_instrument->setEnabled(enabled);
    gActions->store_in_library_instrument->setEnabled(enabled);
}
